@@ -10,7 +10,7 @@
 
 #include "blapi.h"
 
-#define FREEBL_VERSION 0x0323
+#define FREEBL_VERSION 0x0326
 
 struct FREEBLVectorStr {
 
@@ -812,6 +812,104 @@ struct FREEBLVectorStr {
 
     /* Version 3.023 came to here */
 
+    PRBool (*p_KEA_PrimeCheck)(SECItem *prime);
+    /* Version 3.024 came to here */
+
+    SECStatus (*p_ChaCha20_InitContext)(ChaCha20Context *ctx,
+                                        const unsigned char *key,
+                                        unsigned int keyLen,
+                                        const unsigned char *nonce,
+                                        unsigned int nonceLen,
+                                        PRUint32 ctr);
+
+    ChaCha20Context *(*p_ChaCha20_CreateContext)(const unsigned char *key,
+                                                 unsigned int keyLen,
+                                                 const unsigned char *nonce,
+                                                 unsigned int nonceLen,
+                                                 PRUint32 ctr);
+
+    void (*p_ChaCha20_DestroyContext)(ChaCha20Context *ctx, PRBool freeit);
+
+    /* Version 3.025 came to here */
+
+    SHA3_224Context *(*p_SHA3_224_NewContext)(void);
+    void (*p_SHA3_224_DestroyContext)(SHA3_224Context *cx, PRBool freeit);
+    unsigned int (*p_SHA3_224_FlattenSize)(SHA3_224Context *cx);
+    void (*p_SHA3_224_Begin)(SHA3_224Context *cx);
+    void (*p_SHA3_224_Update)(SHA3_224Context *cx, const unsigned char *input,
+                              unsigned int inputLen);
+    void (*p_SHA3_224_End)(SHA3_224Context *cx, unsigned char *digest,
+                           unsigned int *digestLen, unsigned int maxDigestLen);
+
+    SECStatus (*p_SHA3_224_HashBuf)(unsigned char *dest, const unsigned char *src,
+                                    PRUint32 src_length);
+    SECStatus (*p_SHA3_224_Hash)(unsigned char *dest, const char *src);
+
+    SHA3_256Context *(*p_SHA3_256_NewContext)(void);
+    void (*p_SHA3_256_DestroyContext)(SHA3_256Context *cx, PRBool freeit);
+    unsigned int (*p_SHA3_256_FlattenSize)(SHA3_256Context *cx);
+    void (*p_SHA3_256_Begin)(SHA3_256Context *cx);
+    void (*p_SHA3_256_Update)(SHA3_256Context *cx, const unsigned char *input,
+                              unsigned int inputLen);
+    void (*p_SHA3_256_End)(SHA3_256Context *cx, unsigned char *digest,
+                           unsigned int *digestLen, unsigned int maxDigestLen);
+
+    SECStatus (*p_SHA3_256_HashBuf)(unsigned char *dest, const unsigned char *src,
+                                    PRUint32 src_length);
+    SECStatus (*p_SHA3_256_Hash)(unsigned char *dest, const char *src);
+
+    SHA3_384Context *(*p_SHA3_384_NewContext)(void);
+    void (*p_SHA3_384_DestroyContext)(SHA3_384Context *cx, PRBool freeit);
+    unsigned int (*p_SHA3_384_FlattenSize)(SHA3_384Context *cx);
+    void (*p_SHA3_384_Begin)(SHA3_384Context *cx);
+    void (*p_SHA3_384_Update)(SHA3_384Context *cx, const unsigned char *input,
+                              unsigned int inputLen);
+    void (*p_SHA3_384_End)(SHA3_384Context *cx, unsigned char *digest,
+                           unsigned int *digestLen, unsigned int maxDigestLen);
+
+    SECStatus (*p_SHA3_384_HashBuf)(unsigned char *dest, const unsigned char *src,
+                                    PRUint32 src_length);
+    SECStatus (*p_SHA3_384_Hash)(unsigned char *dest, const char *src);
+
+    SHA3_512Context *(*p_SHA3_512_NewContext)(void);
+    void (*p_SHA3_512_DestroyContext)(SHA3_512Context *cx, PRBool freeit);
+    unsigned int (*p_SHA3_512_FlattenSize)(SHA3_512Context *cx);
+    void (*p_SHA3_512_Begin)(SHA3_512Context *cx);
+    void (*p_SHA3_512_Update)(SHA3_512Context *cx, const unsigned char *input,
+                              unsigned int inputLen);
+    void (*p_SHA3_512_End)(SHA3_512Context *cx, unsigned char *digest,
+                           unsigned int *digestLen, unsigned int maxDigestLen);
+
+    SECStatus (*p_SHA3_512_HashBuf)(unsigned char *dest, const unsigned char *src,
+                                    PRUint32 src_length);
+    SECStatus (*p_SHA3_512_Hash)(unsigned char *dest, const char *src);
+
+    SHAKE_128Context *(*p_SHAKE_128_NewContext)(void);
+    void (*p_SHAKE_128_DestroyContext)(SHAKE_128Context *cx, PRBool freeit);
+    void (*p_SHAKE_128_Begin)(SHAKE_128Context *cx);
+    void (*p_SHAKE_128_Absorb)(SHAKE_128Context *cx, const unsigned char *input,
+                               unsigned int inputLen);
+    void (*p_SHAKE_128_SqueezeEnd)(SHAKE_128Context *cx, unsigned char *digest,
+                                   unsigned int digestLen);
+
+    SECStatus (*p_SHAKE_128_HashBuf)(unsigned char *dest, PRUint32 dest_length,
+                                     const unsigned char *src, PRUint32 src_length);
+    SECStatus (*p_SHAKE_128_Hash)(unsigned char *dest, PRUint32 dest_length, const char *src);
+
+    SHAKE_256Context *(*p_SHAKE_256_NewContext)(void);
+    void (*p_SHAKE_256_DestroyContext)(SHAKE_256Context *cx, PRBool freeit);
+    void (*p_SHAKE_256_Begin)(SHAKE_256Context *cx);
+    void (*p_SHAKE_256_Absorb)(SHAKE_256Context *cx, const unsigned char *input,
+                               unsigned int inputLen);
+    void (*p_SHAKE_256_SqueezeEnd)(SHAKE_256Context *cx, unsigned char *digest,
+                                   unsigned int digestLen);
+
+    SECStatus (*p_SHAKE_256_HashBuf)(unsigned char *dest, PRUint32 dest_length,
+                                     const unsigned char *src, PRUint32 src_length);
+    SECStatus (*p_SHAKE_256_Hash)(unsigned char *dest, PRUint32 dest_length, const char *src);
+
+    /* Version 3.026 came to here */
+
     /* Add new function pointers at the end of this struct and bump
      * FREEBL_VERSION at the beginning of this file. */
 };
@@ -888,3 +986,29 @@ typedef SECStatus (*F_SEED_Decrypt)(SEEDContext *cx, unsigned char *output,
                                     unsigned int *outputLen, unsigned int maxOutputLen,
                                     const unsigned char *input, unsigned int inputLen);
 #endif
+
+#ifdef NSS_DISABLE_DEPRECATED_RC2
+typedef RC2Context *(*F_RC2_CreateContext)(const unsigned char *key,
+                                           unsigned int len, const unsigned char *iv,
+                                           int mode, unsigned effectiveKeyLen);
+
+typedef void (*F_RC2_DestroyContext)(RC2Context *cx, PRBool freeit);
+
+typedef SECStatus (*F_RC2_Encrypt)(RC2Context *cx, unsigned char *output,
+                                   unsigned int *outputLen, unsigned int maxOutputLen,
+                                   const unsigned char *input, unsigned int inputLen);
+
+typedef SECStatus (*F_RC2_Decrypt)(RC2Context *cx, unsigned char *output,
+                                   unsigned int *outputLen, unsigned int maxOutputLen,
+                                   const unsigned char *input, unsigned int inputLen);
+
+typedef SECStatus (*F_RC2_InitContext)(RC2Context *cx,
+                                       const unsigned char *key,
+                                       unsigned int keylen,
+                                       const unsigned char *iv,
+                                       int mode,
+                                       unsigned int effectiveKeyLen,
+                                       unsigned int);
+
+typedef RC2Context *(*F_RC2_AllocateContext)(void);
+#endif
