@@ -12,7 +12,7 @@
 #include "pk11pub.h"
 #include "ssl3ext.h"
 #include "ssl3exthandle.h"
-#include "tls13esni.h"
+#include "tls13ech.h"
 #include "tls13exthandle.h"
 #include "tls13psk.h"
 #include "tls13subcerts.h"
@@ -126,7 +126,6 @@ tls13_ClientSendKeyShareXtn(const sslSocket *ss, TLSExtensionData *xtnData,
 {
     SECStatus rv;
     PRCList *cursor;
-    unsigned int extStart;
     unsigned int lengthOffset;
 
     if (ss->vrange.max < SSL_LIBRARY_VERSION_TLS_1_3) {
@@ -138,8 +137,6 @@ tls13_ClientSendKeyShareXtn(const sslSocket *ss, TLSExtensionData *xtnData,
     SSL_TRC(3, ("%d: TLS13[%d]: send client key share xtn",
                 SSL_GETPID(), ss->fd));
 
-    extStart = SSL_BUFFER_LEN(buf);
-
     /* Save the offset to the length. */
     rv = sslBuffer_Skip(buf, 2, &lengthOffset);
     if (rv != SECSuccess) {
@@ -157,14 +154,30 @@ tls13_ClientSendKeyShareXtn(const sslSocket *ss, TLSExtensionData *xtnData,
             return SECFailure;
         }
     }
-    rv = sslBuffer_InsertLength(buf, lengthOffset, 2);
-    if (rv != SECSuccess) {
-        return SECFailure;
+
+    /* GREASE KeyShareEntry:
+     * [The client] MAY also send KeyShareEntry values for a subset of those
+     * selected in the "key_share" extension.  For each of these, the
+     * "key_exchange" field MAY be any value [RFC8701, Section 3.1].
+     *
+     * By default we do not send KeyShares for every NamedGroup so the
+     * ServerKeyShare handshake message / additional round-trip is not
+     * triggered by sending GREASE KeyShareEntries. */
+    if (ss->opt.enableGrease) {
+        rv = sslBuffer_AppendNumber(buf, ss->ssl3.hs.grease->idx[grease_group], 2);
+        if (rv != SECSuccess)
+            return rv;
+        /* Entry length */
+        rv = sslBuffer_AppendNumber(buf, 2, 2);
+        if (rv != SECSuccess)
+            return rv;
+        /* Entry value */
+        rv = sslBuffer_AppendNumber(buf, 0xCD, 2);
+        if (rv != SECSuccess)
+            return rv;
     }
 
-    rv = SECITEM_MakeItem(NULL, &xtnData->keyShareExtension,
-                          SSL_BUFFER_BASE(buf) + extStart,
-                          SSL_BUFFER_LEN(buf) - extStart);
+    rv = sslBuffer_InsertLength(buf, lengthOffset, 2);
     if (rv != SECSuccess) {
         return SECFailure;
     }
@@ -433,6 +446,19 @@ tls13_ClientSendPreSharedKeyXtn(const sslSocket *ss, TLSExtensionData *xtnData,
         return SECSuccess;
     }
 
+    /* ...or if PSKs are incompatible with negotiated ciphersuites
+     * (different hash algorithms) on HRR.
+     *
+     * In addition, in its updated ClientHello, the client SHOULD NOT offer any
+     * pre-shared keys associated with a hash other than that of the selected
+     * cipher suite.  This allows the client to avoid having to compute partial
+     * hash transcripts for multiple hashes in the second ClientHello
+     * [RFC8446, Section 4.1.4]. */
+    if (ss->ssl3.hs.helloRetry &&
+        (psk->hash != ss->ssl3.hs.suite_def->prf_hash)) {
+        return SECSuccess;
+    }
+
     /* Save where this extension starts so that if we have to add padding, it
     * can be inserted before this extension. */
     PORT_Assert(buf->len >= 4);
@@ -849,6 +875,7 @@ tls13_ClientSendSupportedVersionsXtn(const sslSocket *ss, TLSExtensionData *xtnD
         return SECFailure;
     }
 
+    PORT_Assert(!ss->ssl3.hs.echHpkeCtx || ss->vrange.max >= SSL_LIBRARY_VERSION_TLS_1_3);
     for (version = ss->vrange.max; version >= ss->vrange.min; --version) {
         PRUint16 wire = tls13_EncodeVersion(version,
                                             ss->protocolVariant);
@@ -873,6 +900,16 @@ tls13_ClientSendSupportedVersionsXtn(const sslSocket *ss, TLSExtensionData *xtnD
         }
     }
 
+    /* GREASE SupportedVersions:
+     * A client MAY select one or more GREASE version values and advertise them
+     * in the "supported_versions" extension, if sent [RFC8701, Section 3.1]. */
+    if (ss->opt.enableGrease) {
+        rv = sslBuffer_AppendNumber(buf, ss->ssl3.hs.grease->idx[grease_version], 2);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+    }
+
     rv = sslBuffer_InsertLength(buf, lengthOffset, 1);
     if (rv != SECSuccess) {
         return SECFailure;
@@ -1041,7 +1078,6 @@ SECStatus
 tls13_ClientSendPskModesXtn(const sslSocket *ss, TLSExtensionData *xtnData,
                             sslBuffer *buf, PRBool *added)
 {
-    static const PRUint8 ke_modes[] = { tls13_psk_dh_ke };
     SECStatus rv;
 
     if (ss->vrange.max < SSL_LIBRARY_VERSION_TLS_1_3 ||
@@ -1052,7 +1088,15 @@ tls13_ClientSendPskModesXtn(const sslSocket *ss, TLSExtensionData *xtnData,
     SSL_TRC(3, ("%d: TLS13[%d]: send psk key exchange modes extension",
                 SSL_GETPID(), ss->fd));
 
-    rv = sslBuffer_AppendVariable(buf, ke_modes, sizeof(ke_modes), 1);
+    /* GREASE PskKeyExchangeMode:
+     * A client MAY select one or more GREASE PskKeyExchangeMode values and
+     * advertise them in the "psk_key_exchange_modes" extension, if sent
+     * [RFC8701, Section 3.1]. */
+    if (ss->opt.enableGrease) {
+        rv = sslBuffer_AppendVariable(buf, (PRUint8[]){ tls13_psk_dh_ke, ss->ssl3.hs.grease->pskKem }, 2, 1);
+    } else {
+        rv = sslBuffer_AppendVariable(buf, (PRUint8[]){ tls13_psk_dh_ke }, 1, 1);
+    }
     if (rv != SECSuccess) {
         return SECFailure;
     }
@@ -1172,6 +1216,15 @@ loser:
     return SECFailure;
 }
 
+SECStatus
+tls13_ServerHandleCertAuthoritiesXtn(const sslSocket *ss, TLSExtensionData *xtnData, SECItem *data)
+{
+    SSL_TRC(3, ("%d: TLS13[%d]: ignore certificate_authorities extension",
+                SSL_GETPID(), ss->fd));
+    /* NSS ignores certificate_authorities in the ClientHello */
+    return SECSuccess;
+}
+
 SECStatus
 tls13_ServerSendHrrKeyShareXtn(const sslSocket *ss, TLSExtensionData *xtnData,
                                sslBuffer *buf, PRBool *added)
@@ -1213,289 +1266,93 @@ tls13_ServerSendHrrCookieXtn(const sslSocket *ss, TLSExtensionData *xtnData,
 }
 
 SECStatus
-tls13_ClientSendEsniXtn(const sslSocket *ss, TLSExtensionData *xtnData,
-                        sslBuffer *buf, PRBool *added)
+tls13_ClientHandleHrrEchXtn(const sslSocket *ss, TLSExtensionData *xtnData,
+                            SECItem *data)
 {
-    SECStatus rv;
-    PRUint8 sniBuf[1024];
-    PRUint8 hash[64];
-    sslBuffer sni = SSL_BUFFER(sniBuf);
-    const ssl3CipherSuiteDef *suiteDef;
-    ssl3KeyMaterial keyMat;
-    PRUint8 outBuf[1024];
-    unsigned int outLen;
-    unsigned int sniStart;
-    unsigned int sniLen;
-    sslBuffer aadInput = SSL_BUFFER_EMPTY;
-    unsigned int keyShareBufStart;
-    unsigned int keyShareBufLen;
-
-    PORT_Memset(&keyMat, 0, sizeof(keyMat));
-
-    if (!ss->xtnData.esniPrivateKey) {
+    if (data->len != TLS13_ECH_SIGNAL_LEN) {
+        ssl3_ExtSendAlert(ss, alert_fatal, decode_error);
+        PORT_SetError(SSL_ERROR_RX_MALFORMED_ECH_EXTENSION);
+        return SECFailure;
+    }
+    if (!ssl3_ExtensionAdvertised(ss, ssl_tls13_encrypted_client_hello_xtn)) {
+        ssl3_ExtSendAlert(ss, alert_fatal, illegal_parameter);
+        PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
+        return SECFailure;
+    }
+    if (!ss->ssl3.hs.echHpkeCtx) {
+        SSL_TRC(50, ("%d: TLS13[%d]: client received GREASEd ECH confirmation",
+                     SSL_GETPID(), ss->fd));
         return SECSuccess;
     }
-
-    /* nonce */
-    rv = PK11_GenerateRandom(
-        (unsigned char *)xtnData->esniNonce, sizeof(xtnData->esniNonce));
-    if (rv != SECSuccess) {
+    SSL_TRC(50, ("%d: TLS13[%d]: client received HRR ECH confirmation",
+                 SSL_GETPID(), ss->fd));
+    PORT_Assert(!xtnData->ech);
+    xtnData->ech = PORT_ZNew(sslEchXtnState);
+    if (!xtnData->ech) {
         return SECFailure;
     }
-    rv = sslBuffer_Append(&sni, xtnData->esniNonce, sizeof(xtnData->esniNonce));
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    /* sni */
-    sniStart = SSL_BUFFER_LEN(&sni);
-    rv = ssl3_ClientFormatServerNameXtn(ss, ss->url, xtnData, &sni);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    sniLen = SSL_BUFFER_LEN(&sni) - sniStart;
-    /* Padding. */
-    if (ss->esniKeys->paddedLength > sniLen) {
-        unsigned int paddingRequired = ss->esniKeys->paddedLength - sniLen;
-        while (paddingRequired--) {
-            rv = sslBuffer_AppendNumber(&sni, 0, 1);
-            if (rv != SECSuccess) {
-                return SECFailure;
-            }
-        }
-    }
-
-    suiteDef = ssl_LookupCipherSuiteDef(xtnData->esniSuite);
-    PORT_Assert(suiteDef);
-    if (!suiteDef) {
-        PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
-        return SECFailure;
-    }
-
-    /* Format the first part of the extension so we have the
-     * encoded KeyShareEntry. */
-    rv = sslBuffer_AppendNumber(buf, xtnData->esniSuite, 2);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-    keyShareBufStart = SSL_BUFFER_LEN(buf);
-    rv = tls13_EncodeKeyShareEntry(buf,
-                                   xtnData->esniPrivateKey->group->name,
-                                   xtnData->esniPrivateKey->keys->pubKey);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-    keyShareBufLen = SSL_BUFFER_LEN(buf) - keyShareBufStart;
-
-    if (tls13_GetHashSizeForHash(suiteDef->prf_hash) > sizeof(hash)) {
-        PORT_Assert(PR_FALSE);
-        PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
-        return SECFailure;
-    }
-
-    rv = PK11_HashBuf(ssl3_HashTypeToOID(suiteDef->prf_hash),
-                      hash,
-                      ss->esniKeys->data.data,
-                      ss->esniKeys->data.len);
-    if (rv != SECSuccess) {
-        PORT_Assert(PR_FALSE);
-        return SECFailure;
-    }
-
-    rv = sslBuffer_AppendVariable(buf, hash,
-                                  tls13_GetHashSizeForHash(suiteDef->prf_hash), 2);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    /* Compute the ESNI keys. */
-    rv = tls13_ComputeESNIKeys(ss, xtnData->peerEsniShare,
-                               xtnData->esniPrivateKey->keys,
-                               suiteDef,
-                               hash,
-                               SSL_BUFFER_BASE(buf) + keyShareBufStart,
-                               keyShareBufLen,
-                               CONST_CAST(PRUint8, ss->ssl3.hs.client_random),
-                               &keyMat);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    rv = tls13_FormatEsniAADInput(&aadInput,
-                                  xtnData->keyShareExtension.data,
-                                  xtnData->keyShareExtension.len);
-    if (rv != SECSuccess) {
-        ssl_DestroyKeyMaterial(&keyMat);
-        return SECFailure;
-    }
-
-    /* Now encrypt. */
-    unsigned char *aad = SSL_BUFFER_BASE(&aadInput);
-    int aadLen = SSL_BUFFER_LEN(&aadInput);
-    const ssl3BulkCipherDef *cipher_def = ssl_GetBulkCipherDef(suiteDef);
-    int ivLen = cipher_def->iv_size + cipher_def->explicit_nonce_size;
-    unsigned char zero[sizeof(sslSequenceNumber)] = { 0 };
-    SSLCipherAlgorithm calg = cipher_def->calg;
-    SECItem null_params = { siBuffer, NULL, 0 };
-    PK11Context *ctxt = PK11_CreateContextBySymKey(ssl3_Alg2Mech(calg),
-                                                   CKA_NSS_MESSAGE | CKA_ENCRYPT,
-                                                   keyMat.key, &null_params);
-    if (!ctxt) {
-        ssl_DestroyKeyMaterial(&keyMat);
-        sslBuffer_Clear(&aadInput);
-        return SECFailure;
-    }
-
-    /* This function is a single shot, with fresh/unique keys, no need to
-     * generate the IV internally */
-    rv = tls13_AEAD(ctxt, PR_FALSE /* Encrypt */, CKG_NO_GENERATE, 0,
-                    keyMat.iv, NULL, ivLen, zero, sizeof(zero), aad, aadLen,
-                    outBuf, &outLen, sizeof(outBuf), cipher_def->tag_size,
-                    SSL_BUFFER_BASE(&sni), SSL_BUFFER_LEN(&sni));
-    ssl_DestroyKeyMaterial(&keyMat);
-    sslBuffer_Clear(&aadInput);
-    PK11_DestroyContext(ctxt, PR_TRUE);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    /* Encode the rest. */
-    rv = sslBuffer_AppendVariable(buf, outBuf, outLen, 2);
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    *added = PR_TRUE;
-    return SECSuccess;
-}
-
-static SECStatus
-tls13_ServerSendEsniXtn(const sslSocket *ss, TLSExtensionData *xtnData,
-                        sslBuffer *buf, PRBool *added)
-{
-    SECStatus rv;
-
-    rv = sslBuffer_Append(buf, xtnData->esniNonce, sizeof(xtnData->esniNonce));
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    *added = PR_TRUE;
+    xtnData->ech->hrrConfirmation = data->data;
     return SECSuccess;
 }
 
 SECStatus
-tls13_ServerHandleEsniXtn(const sslSocket *ss, TLSExtensionData *xtnData,
-                          SECItem *data)
+tls13_ClientHandleEchXtn(const sslSocket *ss, TLSExtensionData *xtnData,
+                         SECItem *data)
 {
-    sslReadBuffer buf;
-    PRUint8 *plainText = NULL;
-    unsigned int ptLen;
     SECStatus rv;
+    PRCList parsedConfigs;
+    PR_INIT_CLIST(&parsedConfigs);
 
-    /* If we are doing < TLS 1.3, then ignore this. */
-    if (ss->version < SSL_LIBRARY_VERSION_TLS_1_3) {
-        return SECSuccess;
-    }
-
-    if (!ss->esniKeys) {
-        /* Apparently we used to be configured for ESNI, but
-         * no longer. This violates the spec, or the client is
-         * broken. */
+    /* The [retry config] response is valid only when the server used the
+     * ClientHelloOuter. If the server sent this extension in response to the
+     * inner variant [ECH was accepted], then the client MUST abort with an
+     * "unsupported_extension" alert [draft-ietf-tls-esni-14, Section 5]. */
+    if (ss->ssl3.hs.echAccepted) {
+        PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
+        ssl3_ExtSendAlert(ss, alert_fatal, unsupported_extension);
         return SECFailure;
     }
 
-    plainText = PORT_ZAlloc(data->len);
-    if (!plainText) {
-        return SECFailure;
-    }
-    rv = tls13_ServerDecryptEsniXtn(ss, data->data, data->len,
-                                    plainText, &ptLen, data->len);
-    if (rv) {
-        goto loser;
-    }
-
-    /* Read out the interior extension. */
-    sslReader sniRdr = SSL_READER(plainText, ptLen);
-
-    rv = sslRead_Read(&sniRdr, sizeof(xtnData->esniNonce), &buf);
-    if (rv != SECSuccess) {
-        goto loser;
-    }
-    PORT_Memcpy(xtnData->esniNonce, buf.buf, sizeof(xtnData->esniNonce));
-
-    /* We need to capture the whole block with the length. */
-    SECItem sniItem = { siBuffer, (unsigned char *)SSL_READER_CURRENT(&sniRdr), 0 };
-    rv = sslRead_ReadVariable(&sniRdr, 2, &buf);
-    if (rv != SECSuccess) {
-        goto loser;
-    }
-    sniItem.len = buf.len + 2;
-
-    /* Check the padding. Note we don't need to do this in constant time
-     * because it's inside the AEAD boundary. */
-    /* TODO(ekr@rtfm.com): check that the padding is the right length. */
-    PRUint64 tmp;
-    while (SSL_READER_REMAINING(&sniRdr)) {
-        rv = sslRead_ReadNumber(&sniRdr, 1, &tmp);
-        if (rv != SECSuccess) {
-            goto loser;
+    /* If the server is configured with any ECHConfigs, it MUST include the
+     * "encrypted_client_hello" extension in its EncryptedExtensions with the
+     * "retry_configs" field set to one or more ECHConfig structures with
+     * up-to-date keys [draft-ietf-tls-esni-14, Section 7.1]. */
+    if (ss->ssl3.hs.msg_type != ssl_hs_encrypted_extensions) {
+        PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
+        if (ss->version < SSL_LIBRARY_VERSION_TLS_1_3) {
+            /* For TLS < 1.3 the extension is unkown/unsupported. */
+            ssl3_ExtSendAlert(ss, alert_fatal, unsupported_extension);
+        } else {
+            /* For TLS 1.3 the extension is known but prohibited outside EE
+             * (see RFC8446, Section 4.2 for alert rationale). */
+            ssl3_ExtSendAlert(ss, alert_fatal, illegal_parameter);
         }
-        if (tmp != 0) {
-            goto loser;
-        }
-    }
-
-    rv = ssl3_HandleServerNameXtn(ss, xtnData, &sniItem);
-    if (rv != SECSuccess) {
-        goto loser;
-    }
-
-    rv = ssl3_RegisterExtensionSender(ss, xtnData,
-                                      ssl_tls13_encrypted_sni_xtn,
-                                      tls13_ServerSendEsniXtn);
-    if (rv != SECSuccess) {
-        goto loser;
-    }
-
-    /* Keep track of negotiated extensions. */
-    xtnData->negotiated[xtnData->numNegotiated++] =
-        ssl_tls13_encrypted_sni_xtn;
-
-    PORT_ZFree(plainText, data->len);
-    return SECSuccess;
-loser:
-    PORT_ZFree(plainText, data->len);
-    return SECFailure;
-}
-
-/* Function to check the extension. We don't install a handler here
- * because we need to check for the presence of the extension as
- * well and it's easier to do it in one place. */
-SECStatus
-tls13_ClientCheckEsniXtn(sslSocket *ss)
-{
-    TLSExtension *esniExtension =
-        ssl3_FindExtension(ss, ssl_tls13_encrypted_sni_xtn);
-    if (!esniExtension) {
-        FATAL_ERROR(ss, SSL_ERROR_MISSING_ESNI_EXTENSION, missing_extension);
         return SECFailure;
     }
 
-    if (esniExtension->data.len != sizeof(ss->xtnData.esniNonce)) {
-        FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_ESNI_EXTENSION, illegal_parameter);
+    PORT_Assert(!xtnData->ech);
+    xtnData->ech = PORT_ZNew(sslEchXtnState);
+    if (!xtnData->ech) {
         return SECFailure;
     }
 
-    if (0 != NSS_SecureMemcmp(esniExtension->data.data,
-                              ss->xtnData.esniNonce,
-                              sizeof(ss->xtnData.esniNonce))) {
-        FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_ESNI_EXTENSION, illegal_parameter);
+    /* Parse the list to determine 1) That the configs are valid
+     * and properly encoded, and 2) If any are compatible. */
+    rv = tls13_DecodeEchConfigs(data, &parsedConfigs);
+    if (rv == SECFailure) {
+        ssl3_ExtSendAlert(ss, alert_fatal, decode_error);
+        PORT_SetError(SSL_ERROR_RX_MALFORMED_ECH_CONFIG);
         return SECFailure;
     }
+    /* Don't mark ECH negotiated on rejection with retry_config.
+     * Save the the raw configs so the application can retry. If
+     * we sent GREASE ECH (no echHpkeCtx), don't apply retry_configs. */
+    if (ss->ssl3.hs.echHpkeCtx && !PR_CLIST_IS_EMPTY(&parsedConfigs)) {
+        rv = SECITEM_CopyItem(NULL, &xtnData->ech->retryConfigs, data);
+    }
+    tls13_DestroyEchConfigs(&parsedConfigs);
 
-    return SECSuccess;
+    return rv;
 }
 
 /* Indicates support for the delegated credentials extension. This should be
@@ -1533,7 +1390,8 @@ tls13_ClientSendDelegatedCredentialsXtn(const sslSocket *ss,
         return SECSuccess;
     }
 
-    rv = ssl3_EncodeFilteredSigAlgs(ss, filtered, filteredCount, buf);
+    rv = ssl3_EncodeFilteredSigAlgs(ss, filtered, filteredCount,
+                                    PR_FALSE /* GREASE */, buf);
     if (rv != SECSuccess) {
         return SECFailure;
     }
@@ -1691,3 +1549,254 @@ tls13_ServerHandleDelegatedCredentialsXtn(const sslSocket *ss,
         ss, xtnData, ssl_delegated_credentials_xtn,
         tls13_ServerSendDelegatedCredentialsXtn);
 }
+
+/* Adds the ECH extension containing server retry_configs */
+SECStatus
+tls13_ServerSendEchXtn(const sslSocket *ss,
+                       TLSExtensionData *xtnData,
+                       sslBuffer *buf, PRBool *added)
+{
+    SECStatus rv;
+    PORT_Assert(ss->version >= SSL_LIBRARY_VERSION_TLS_1_3);
+    if (PR_CLIST_IS_EMPTY(&ss->echConfigs)) {
+        return SECSuccess;
+    }
+
+    const sslEchConfig *cfg = (sslEchConfig *)PR_LIST_HEAD(&ss->echConfigs);
+    rv = sslBuffer_AppendVariable(buf, cfg->raw.data, cfg->raw.len, 2);
+    if (rv != SECSuccess) {
+        return SECFailure;
+    }
+
+    *added = PR_TRUE;
+    return SECSuccess;
+}
+
+/* If an ECH server sends the HRR ECH extension after it accepted ECH, the
+ * extension's payload must be set to 8 zero bytes, these are overwritten with
+ * the accept_confirmation value after the required transcript calculation.
+ * If a client-facing/shared-mode server did not accept ECH when offered in CH
+ * or if ECH GREASE is enabled on the server and a ECH extension was received,
+ * a 8 byte random value is set as the extension's payload
+ * [draft-ietf-tls-esni-14, Section 7].
+ *
+ * Depending on the acceptance of ECH, zero or random bytes are written to
+ * ss->ssl3.hs.greaseEchBuf.buf in tls13con.c/tls13_SendHelloRetryRequest(). */
+SECStatus
+tls13_ServerSendHrrEchXtn(const sslSocket *ss, TLSExtensionData *xtnData,
+                          sslBuffer *buf, PRBool *added)
+{
+    SECStatus rv;
+    /* Do not send HRR ECH extension if TLS < 1.3 was negotiated OR no ECH
+     * extension was received OR the server is NOT in any ECH server mode AND
+     * ECH GREASE is NOT enabled. */
+    if (ss->version < SSL_LIBRARY_VERSION_TLS_1_3 ||
+        !xtnData->ech ||
+        (!ss->echPubKey && !ss->opt.enableTls13BackendEch && !ss->opt.enableTls13GreaseEch)) {
+        SSL_TRC(100, ("%d: TLS13[%d]: server not sending HRR ECH Xtn",
+                      SSL_GETPID(), ss->fd));
+        return SECSuccess;
+    }
+    SSL_TRC(100, ("%d: TLS13[%d]: server sending HRR ECH Xtn",
+                  SSL_GETPID(), ss->fd));
+    PR_ASSERT(SSL_BUFFER_LEN(&ss->ssl3.hs.greaseEchBuf) == TLS13_ECH_SIGNAL_LEN);
+    PRINT_BUF(100, (ss, "grease_ech_confirmation", ss->ssl3.hs.greaseEchBuf.buf, TLS13_ECH_SIGNAL_LEN));
+    rv = sslBuffer_AppendBuffer(buf, &ss->ssl3.hs.greaseEchBuf);
+    if (rv != SECSuccess) {
+        return SECFailure;
+    }
+    *added = PR_TRUE;
+    return SECSuccess;
+}
+
+SECStatus
+tls13_ServerHandleInnerEchXtn(const sslSocket *ss, TLSExtensionData *xtnData,
+                              SECItem *data)
+{
+    PRUint64 xtn_type;
+    sslReader xtnReader = SSL_READER(data->data, data->len);
+
+    PR_ASSERT(ss->ssl3.hs.echAccepted || ss->opt.enableTls13BackendEch);
+    PR_ASSERT(!xtnData->ech->receivedInnerXtn);
+
+    SECStatus rv = sslRead_ReadNumber(&xtnReader, 1, &xtn_type);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+    if (xtn_type != ech_xtn_type_inner) {
+        goto alert_loser;
+    }
+    if (SSL_READER_REMAINING(&xtnReader)) {
+        /* Inner ECH Extension must contain only type enum */
+        goto alert_loser;
+    }
+
+    xtnData->ech->receivedInnerXtn = PR_TRUE;
+    xtnData->negotiated[xtnData->numNegotiated++] = ssl_tls13_encrypted_client_hello_xtn;
+    return SECSuccess;
+
+alert_loser:
+    ssl3_ExtSendAlert(ss, alert_fatal, decode_error);
+    PORT_SetError(SSL_ERROR_RX_MALFORMED_ECH_EXTENSION);
+    return SECFailure;
+}
+
+SECStatus
+tls13_ServerHandleOuterEchXtn(const sslSocket *ss, TLSExtensionData *xtnData,
+                              SECItem *data)
+{
+    SECStatus rv;
+    HpkeKdfId kdf;
+    HpkeAeadId aead;
+    PRUint32 tmp;
+    PRUint8 configId;
+    SECItem senderPubKey;
+    SECItem encryptedCh;
+
+    PRUint32 xtn_type;
+    rv = ssl3_ExtConsumeHandshakeNumber(ss, &xtn_type, 1, &data->data, &data->len);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+    if (xtn_type != ech_xtn_type_outer && xtn_type != ech_xtn_type_inner) {
+        SSL_TRC(3, ("%d: TLS13[%d]: unexpected ECH extension type in client hello outer, alert",
+                    SSL_GETPID(), ss->fd));
+        goto alert_loser;
+    }
+    /* If we are operating in shared mode, we can accept an inner xtn in the ClientHelloOuter */
+    if (xtn_type == ech_xtn_type_inner) {
+        if (!ss->opt.enableTls13BackendEch) {
+            ssl3_ExtSendAlert(ss, alert_fatal, illegal_parameter);
+            PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
+            return SECFailure;
+        }
+        PORT_Assert(!xtnData->ech);
+        xtnData->ech = PORT_ZNew(sslEchXtnState);
+        if (!xtnData->ech) {
+            return SECFailure;
+        }
+        /* We have to rewind the buffer advanced by ssl3_ExtConsumeHandshakeNumber */
+        data->data--;
+        data->len++;
+        return tls13_ServerHandleInnerEchXtn(ss, xtnData, data);
+    }
+    if (ss->ssl3.hs.echAccepted) {
+        ssl3_ExtSendAlert(ss, alert_fatal, illegal_parameter);
+        PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
+        return SECFailure;
+    }
+
+    SSL_TRC(3, ("%d: TLS13[%d]: handle outer ECH extension",
+                SSL_GETPID(), ss->fd));
+
+    PORT_Assert(!xtnData->ech);
+    xtnData->ech = PORT_ZNew(sslEchXtnState);
+    if (!xtnData->ech) {
+        return SECFailure;
+    }
+
+    /* Parse the KDF and AEAD. */
+    rv = ssl3_ExtConsumeHandshakeNumber(ss, &tmp, 2,
+                                        &data->data, &data->len);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+    kdf = (HpkeKdfId)tmp;
+    rv = ssl3_ExtConsumeHandshakeNumber(ss, &tmp, 2,
+                                        &data->data, &data->len);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+    aead = (HpkeAeadId)tmp;
+
+    /* config_id */
+    rv = ssl3_ExtConsumeHandshakeNumber(ss, &tmp, 1,
+                                        &data->data, &data->len);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+    configId = tmp;
+
+    /* enc */
+    rv = ssl3_ExtConsumeHandshakeVariable(ss, &senderPubKey, 2,
+                                          &data->data, &data->len);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+
+    /* payload, which must be final and non-empty. */
+    xtnData->ech->payloadStart = data->data + 2; /* Move past length */
+    rv = ssl3_ExtConsumeHandshakeVariable(ss, &encryptedCh, 2,
+                                          &data->data, &data->len);
+    if (rv != SECSuccess) {
+        goto alert_loser;
+    }
+    if (data->len || !encryptedCh.len) {
+        goto alert_loser;
+    }
+
+    if (!ss->ssl3.hs.helloRetry) {
+        /* In the real ECH HRR case, config_id and enc should be empty. This
+         * is checked after acceptance, because it might be GREASE ECH. */
+        if (!senderPubKey.len) {
+            goto alert_loser;
+        }
+
+        rv = SECITEM_CopyItem(NULL, &xtnData->ech->senderPubKey, &senderPubKey);
+        if (rv == SECFailure) {
+            return SECFailure;
+        }
+    }
+
+    rv = SECITEM_CopyItem(NULL, &xtnData->ech->innerCh, &encryptedCh);
+    PRINT_BUF(100, (ss, "CT for ECH Decryption", encryptedCh.data, encryptedCh.len));
+    if (rv == SECFailure) {
+        return SECFailure;
+    }
+    xtnData->ech->configId = configId;
+    xtnData->ech->kdfId = kdf;
+    xtnData->ech->aeadId = aead;
+
+    /* Not negotiated until tls13_MaybeAcceptEch. */
+    return SECSuccess;
+
+alert_loser:
+    ssl3_ExtSendAlert(ss, alert_fatal, decode_error);
+    PORT_SetError(SSL_ERROR_RX_MALFORMED_ECH_EXTENSION);
+    return SECFailure;
+}
+
+SECStatus
+tls13_SendEmptyGreaseXtn(const sslSocket *ss,
+                         TLSExtensionData *xtnData,
+                         sslBuffer *buf, PRBool *added)
+{
+    if (!ss->opt.enableGrease ||
+        (!ss->sec.isServer && ss->vrange.max < SSL_LIBRARY_VERSION_TLS_1_3) ||
+        (ss->sec.isServer && ss->version < SSL_LIBRARY_VERSION_TLS_1_3)) {
+        return SECSuccess;
+    }
+
+    *added = PR_TRUE;
+    return SECSuccess;
+}
+
+SECStatus
+tls13_SendGreaseXtn(const sslSocket *ss,
+                    TLSExtensionData *xtnData,
+                    sslBuffer *buf, PRBool *added)
+{
+    if (!ss->opt.enableGrease ||
+        (!ss->sec.isServer && ss->vrange.max < SSL_LIBRARY_VERSION_TLS_1_3) ||
+        (ss->sec.isServer && ss->version < SSL_LIBRARY_VERSION_TLS_1_3)) {
+        return SECSuccess;
+    }
+
+    SECStatus rv = sslBuffer_AppendVariable(buf, (PRUint8[]){ 0x00 }, 1, 2);
+    if (rv != SECSuccess) {
+        return SECFailure;
+    }
+
+    *added = PR_TRUE;
+    return SECSuccess;
+}
